var _a;
import { __decorate } from "tslib";
import { attr } from '@microsoft/fast-element';
import { FoundationElement } from '@microsoft/fast-foundation';
import { componentType, getParentLayoutComponent, layoutRegionValue, wrapperTemplate, } from '../utils';
import { logger } from '../utils/logger';
const defaultLayoutType = layoutRegionValue[0];
const regionConveter = (x) => {
    const t = {
        horizontal: 'column',
        vertical: 'row',
        tabs: 'stack',
    };
    return t[x];
};
/**
 * @public
 * `FoundationLayoutRegion` is a custom element that represents a region in the layout.
 * @remarks
 * This element is used to create a region in the layout. It can be used to create a horizontal or vertical split, or a tabbed region.
 *
 * This is a simple component which is only used to define the layout splits; any JavaScript API interactions or custom styling is used via {@link FoundationLayout}.
 * @tagname %%prefix%%-layout-region
 */
export class FoundationLayoutRegion extends FoundationElement {
    constructor() {
        super(...arguments);
        /** @internal */
        this[_a] = 'horizontal';
    }
    /** @internal */
    connectedCallback() {
        super.connectedCallback();
        if (!layoutRegionValue.includes(this.type)) {
            this.type = defaultLayoutType;
            logger.warn(`Using invalid attribute "type", defaulting to ${defaultLayoutType}`);
        }
        this[componentType] = this.type;
        this.setupLayoutRegion();
    }
    /**
     * Gets the layout config for this region using {@link FoundationLayoutRegion.constructLayoutConfig} and adds it into the main config tree via the parents. It saves a reference to the this config so a child can add its config in later.
     * @internal
     */
    setupLayoutRegion() {
        this.layoutConfig = this.getParentLayoutComponent().addItemFromChild(this.constructLayoutConfig());
    }
    /**
     * Creates a golden layout config for this object.
     * @internal
     */
    constructLayoutConfig() {
        const glRegion = regionConveter(this[componentType]);
        return {
            type: glRegion,
            content: [],
            size: this.size,
        };
    }
    /** @internal */
    getParentLayoutComponent() {
        return getParentLayoutComponent.apply(this);
    }
    /** @internal */
    addItemFromChild(item) {
        this.layoutConfig.content.push(item);
        return this.layoutConfig.content[this.layoutConfig.content.length - 1];
    }
    /** @internal */
    requestLayoutReload() {
        this.getParentLayoutComponent().requestLayoutReload();
    }
    /** @internal */
    cacheElementsAndRegister(config) {
        return this.getParentLayoutComponent().cacheElementsAndRegister(config);
    }
}
_a = componentType;
__decorate([
    attr
], FoundationLayoutRegion.prototype, "size", void 0);
__decorate([
    attr
], FoundationLayoutRegion.prototype, "type", void 0);
/**
 * The FoundationLayoutRegion Composed element
 *
 * @internal
 * @remarks
 * Can be registered with the design system via {@link foundationlayoutComponents}.
 * You do not need to compose your own version of this to update the styles of the layout.
 *
 * Tag Name: <{prefix}-layout-region>
 */
export const foundationLayoutRegion = FoundationLayoutRegion.compose({
    baseName: 'layout-region',
    template: wrapperTemplate,
});
